import * as React from 'react';
import * as redux from 'redux';
import * as classNames from 'classnames';
import { connect as fela, withTheme } from 'react-fela';

import { SettingsTypes } from '../../core';

import { USER_SETTINGS_QUERY, USER_PREFERENCES_QUERY } from '../../graphql';
import { insertNewContent, checkExist } from '../../utils';
import * as componentStyles from './styles';
import { parse } from '../../../../common/jsonc-parser/main';

import WorkspaceCustomSettings from '../WorkspaceCustomSettings';
import UserCustomSettings from '../UserCustomSettings';

export type ICustomSettingsApiProps = {
    userPref?: any;
    saveMutate: (args: any) => any
    updateConfigMutate: () => void
};

export type ICustomSettingsStateProps = {
    activeId: number;
    newLine: string;
    rootPath: string;
};

export type ICustomSettingsDispatchProps = {
    setActiveJsonEditor: (editorId: number) => void;
    change: (changed: boolean) => void;
    setNewLine: (content) => void;
};

export type ICustomSettingsOwnProps = {
    styles: any;
    onKeyDown: any;
    client: any;
    content: any;
    onChange: any;
};

export type ICustomSettingsState = {
    prefs: any;
};

export type ICustomSettings = ICustomSettingsApiProps & ICustomSettingsStateProps & 
ICustomSettingsDispatchProps & ICustomSettingsOwnProps;


class CustomSettingsView extends React.Component<ICustomSettings, ICustomSettingsState> {

    constructor(props) {
        super(props);

        this.state = {
            prefs: null,
        };
    }

    public componentWillMount() {
        const { userPref } = this.props;
        if (userPref) {
            this.setState({ prefs: userPref.prefs });
        }
    }

    public componentWillReceiveProps(nextProps) {
        const { userPref, newLine } = nextProps;
        const { userPref: userPrevPrefs, setNewLine } = this.props;

        if (userPref !== userPrevPrefs) {
            this.setState({ prefs: userPref.prefs });
        }

        if (newLine) {
            const { prefs } = this.state;
            setNewLine(null);
            let newContent = '';
            if (prefs) {
                let value = prefs.newValue && prefs.newValue !== '' ? prefs.newValue : prefs.content;
                const checked = checkExist(value, newLine);
                if (!checked) {
                    newContent = insertNewContent(value, newLine);
                } else {
                    newContent = checked.value;
                }

                this.onChange(newContent);
            } else {
                newContent = `{\n${newLine}\n}`;
                this.props.change(true);
                this.setState({ prefs: { newValue: newContent } });
            }
        }
    }

    private onChange = (newValue) => {
        const { prefs } = this.state;
        if (!prefs.newValue || prefs.newValue && prefs.newValue === '') {
            this.props.change(true);
        }

        this.setState({ prefs: { ...prefs, newValue: newValue } });
    }

    private onUserSettings = () => {
        this.props.setActiveJsonEditor(SettingsTypes.USER);
    }

    private onWorkSpaceSettings = () => {
        this.props.setActiveJsonEditor(SettingsTypes.WORKSPACE);
    }

    private onKeyDown = (e) => {
        const { onKeyDown, client, rootPath } = this.props;

        const { prefs } = this.state;
        let charCode = String.fromCharCode(e.which).toLowerCase();

        if ((e.metaKey || e.ctrlKey) && charCode === 's') {
            e.preventDefault();
            onKeyDown(e, {update: this.props.updateConfigMutate});
            const user = '1';
            if (prefs.newValue) {
                this.props.saveMutate({ variables: { content: prefs.newValue, user } });
                this.setState({ prefs: { ...this.state.prefs, content: prefs.newValue, newValue: '' } });
                let userSettings = {
                    data: null,
                    __typename: 'UserSettings',
                };
                try {
                    const savedSettings = client.readQuery({
                        query: USER_SETTINGS_QUERY,
                        variables: { user, rootpath: rootPath },
                    });
                    userSettings = savedSettings.userSettings;
                } catch (err) {
                    // console.log('err', err);
                }

                const settings = parse(prefs.newValue);
                userSettings.data = JSON.stringify(settings);
                client.writeQuery({
                    query: USER_SETTINGS_QUERY,
                    variables: { user, rootpath: rootPath },
                    data: { userSettings },
                });

                client.writeQuery({
                    query: USER_PREFERENCES_QUERY,
                    data: {
                        userPreferences: {
                            prefs: {
                                content: prefs.newValue,
                                user,
                                __typename: 'IUserPreferences',
                            },
                            __typename: 'UserPreferencesRes',
                        },
                    },
                });
            }
        }

    }

    public render() {
        const { styles, activeId, content, onChange, userPref } = this.props;
        const { prefs } = this.state;
        return (
            <div className={styles.container} onKeyDown={this.onKeyDown}>
                <div className={styles.tabWrapper}>
                    <div
                        className={activeId === 0 ? classNames(styles.activeTab, styles.tab) : styles.tab}
                        onClick={this.onUserSettings}
                    >
                        user serttings
                    </div>
                    <div
                        className={activeId === 1 ? classNames(styles.activeTab, styles.tab) : styles.tab}
                        onClick={this.onWorkSpaceSettings}
                    >
                        workspace settings
                    </div>
                </div>
                {activeId === 0 &&
                    <UserCustomSettings
                        prefs={prefs}
                        onChange={this.onChange}
                    />}
                {activeId === 1 &&
                    <WorkspaceCustomSettings
                        content={content}
                        onChange={onChange}
                    />}
            </div>
        );
    }
}

export const CustomSettings: any = fela({
    container: () => componentStyles.container(),
    tabWrapper: () => componentStyles.tabWrapper(),
    tab: ({ theme }) => componentStyles.tab(theme),
    activeTab: ({ theme }) => componentStyles.activeTab(theme),
})(withTheme(CustomSettingsView));
