import * as React from 'react';
import { connect as fela, withTheme } from 'react-fela';
import { Resizable } from 'react-resizable';

import { MonacoLoader } from '@workbench-stack/monaco-browser';

import { getContent, getDeltaDecorations, addCategories, getPrefsList } from '../../utils';

import HelperPopup from '../HelperPopup';

export type IDefaultSettingsStateProps = {
    activeId: number;
    width: number;
};

export type IDefaultSettingsDispatchProps = {
    resizeWidth: (width) => void;
};

export type IDefaultSettingsOwnProps = {
    styles: any;
    defaultPrefs: any;
    client: any;
    editorFile: any;
    editorContent: any;
    theme: any;
};

export type IDefaultSettingsState = {
    test: number,
    openHint: boolean,
    currentHint: any,
    hintPosition: any,
};

export type IDefaultSettings = IDefaultSettingsStateProps & IDefaultSettingsDispatchProps & IDefaultSettingsOwnProps;

class DefaultSettingsView extends React.Component<IDefaultSettings, IDefaultSettingsState> {
    private monaco: any;
    private editor: any;
    private viewZone: number[];

    constructor(props) {
        super(props);

        this.state = {
            test: 0,
            openHint: false,
            currentHint: null,
            hintPosition: null,
        };
    }

    private onResizeStop = (e, { size: { width } }) => {
        if (width >= 200 && width <= 700) {
            this.props.resizeWidth(width);
        }
    }

    private onResize = (e, { size: { width } }) => {
        if (width >= 200 && width <= 500) {
            this.props.resizeWidth(width);
        }
    }

    private editorDidMount = (editor, monaco) => {
        editor.onDidScrollChange(e => {
            const { openHint } = this.state;
            if (openHint && e.scrollTopChanged) {
                this.onHintHide();
            }
        });

        editor.onMouseDown(e => {
            const { defaultPrefs } = this.props;
            const { openHint } = this.state;
            if (openHint) {
                this.onHintHide();
            } else if (e.target.element.className === 'cgmr line-hint' ||
                e.target.element.parentElement.className === 'margin-view-overlays' ||
                e.target.element.parentElement.className === 'margin-view-overlays focused') {
                const line = e.target.position.lineNumber;
                const prefIndex = Math.ceil(line / 3);
                const prefsList = getPrefsList(defaultPrefs);
                if (line % 3 !== 0 && prefIndex <= prefsList.length) {
                    this.setState({
                        currentHint: prefsList[Math.ceil(line / 3) - 1],
                        openHint: true,
                        hintPosition: e.event.posy,
                    });
                } else {
                    this.onHintHide();
                }
            }
        });

        this.monaco = monaco;
        this.editor = editor;

    }

    private onHintHide = () => {
        this.setState({
            currentHint: null,
            openHint: false,
            hintPosition: null,
        });
    }

    public render() {
        const { styles, defaultPrefs, width, client, editorFile, activeId, editorContent, theme } = this.props;
        const { test, currentHint, openHint, hintPosition } = this.state;

        const editorOptions = {
            selectOnLineNumbers: false,
            readOnly: true,
            wordWrap: 'bounded',
            automaticLayout: true,
            minimap: {
                enabled: false,
            },
            lineNumbers: 'off',
            glyphMargin: true,
            theme: 'vs-dark',
        };
        setTimeout(() => {
            if (!test) {
                this.setState({ test: 1 });
            }
        }, 1500);
        if (!test) { return <div />; }
        const value = defaultPrefs ? getContent(defaultPrefs) : '';
        if (theme) { editorOptions.theme = theme.className; }

        if (this.monaco) {
            const decorations = this.editor.deltaDecorations(
                [], getDeltaDecorations(this.monaco, this.props.defaultPrefs));

            this.viewZone = addCategories(defaultPrefs, this.editor, this.viewZone);
        }

        return (
            <Resizable
                id={'defaultSettings'}
                axis={'x'}
                height={Infinity}
                width={width}
                className={styles.resizable}
                onResize={this.onResize}
                onResizeStop={this.onResizeStop}
            >
                <div className={styles.container}>
                    <MonacoLoader
                        width="100%"
                        height="100%"
                        language="jsonc2"
                        value={value}
                        options={editorOptions}
                        uri="static/path"
                        enableLanguageService={true}
                        editorDidMount={this.editorDidMount}
                    />
                    <HelperPopup
                        preference={currentHint}
                        onHideHints={this.onHintHide}
                        open={openHint}
                        client={client}
                        editorFile={editorFile}
                        activeId={activeId}
                        hintPosition={hintPosition}
                        editorContent={editorContent}
                    />
                </div>
            </Resizable>
        );

    }
}

export const DefaultSettings: any = fela<any, any>({
    container: () => ({
        height: '100%',
        '& .line-hint': {
            background: `url("data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' width='16' height='16'%3E%3Cstyle%3E.icon-canvas-transparent{opacity:0;fill:%232d2d30}.icon-vs-out{fill:%232d2d30}.icon-vs-bg{fill:%23c5c5c5}%3C/style%3E%3Cpath class='icon-canvas-transparent' d='M16 16H0V0h16v16z' id='canvas'/%3E%3Cpath class='icon-vs-bg' d='M14.598 4.25L12.91 6l-3-3 1.688-1.75 3 3zM8.91 4l-7 7 3 3 7-7-3-3zM1 12.09V15h2.91L1 12.09z' id='iconBg'/%3E%3C/svg%3E") 50% no-repeat`,
        },
        '& .widget-title': {
            background: 'gray',
            padding: '0 10px',
        },
    }),
    resizable: () => ({
        position: 'relative',
        '> .react-resizable-handle': {
            position: 'absolute',
            width: '20px',
            height: '100%',
            top: 0,
            right: 0,
            padding: '0 3px 3px 0',
            boxSizing: 'border-box',
            cursor: 'col-resize',
        },
    }),
})(withTheme(DefaultSettingsView));

